package javax.management.modelmbean;

import java.security.AccessController;
import com.sun.jmx.mbeanserver.GetPropertyAction;
import java.io.ObjectOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import javax.management.RuntimeOperationsException;
import javax.management.MBeanException;
import java.util.logging.Level;
import com.sun.jmx.defaults.JmxProperties;
import javax.management.MBeanOperationInfo;
import javax.management.MBeanNotificationInfo;
import javax.management.MBeanConstructorInfo;
import javax.management.MBeanAttributeInfo;
import javax.management.Descriptor;
import java.io.ObjectStreamField;
import javax.management.MBeanInfo;

public class ModelMBeanInfoSupport extends MBeanInfo implements ModelMBeanInfo
{
    private static final long oldSerialVersionUID = -3944083498453227709L;
    private static final long newSerialVersionUID = -1935722590756516193L;
    private static final ObjectStreamField[] oldSerialPersistentFields;
    private static final ObjectStreamField[] newSerialPersistentFields;
    private static final long serialVersionUID;
    private static final ObjectStreamField[] serialPersistentFields;
    private static boolean compat;
    private Descriptor modelMBeanDescriptor;
    private MBeanAttributeInfo[] modelMBeanAttributes;
    private MBeanConstructorInfo[] modelMBeanConstructors;
    private MBeanNotificationInfo[] modelMBeanNotifications;
    private MBeanOperationInfo[] modelMBeanOperations;
    private static final String ATTR = "attribute";
    private static final String OPER = "operation";
    private static final String NOTF = "notification";
    private static final String CONS = "constructor";
    private static final String MMB = "mbean";
    private static final String ALL = "all";
    private static final String currClass = "ModelMBeanInfoSupport";
    private static final ModelMBeanAttributeInfo[] NO_ATTRIBUTES;
    private static final ModelMBeanConstructorInfo[] NO_CONSTRUCTORS;
    private static final ModelMBeanNotificationInfo[] NO_NOTIFICATIONS;
    private static final ModelMBeanOperationInfo[] NO_OPERATIONS;
    
    public ModelMBeanInfoSupport(ModelMBeanInfo modelMBeanInfo) {
        super(modelMBeanInfo.getClassName(), modelMBeanInfo.getDescription(), modelMBeanInfo.getAttributes(), modelMBeanInfo.getConstructors(), modelMBeanInfo.getOperations(), modelMBeanInfo.getNotifications());
        this.modelMBeanDescriptor = null;
        this.modelMBeanAttributes = modelMBeanInfo.getAttributes();
        this.modelMBeanConstructors = modelMBeanInfo.getConstructors();
        this.modelMBeanOperations = modelMBeanInfo.getOperations();
        this.modelMBeanNotifications = modelMBeanInfo.getNotifications();
        try {
            this.modelMBeanDescriptor = this.validDescriptor(modelMBeanInfo.getMBeanDescriptor());
        }
        catch (MBeanException ex) {
            this.modelMBeanDescriptor = this.validDescriptor(null);
            if (JmxProperties.MODELMBEAN_LOGGER.isLoggable(Level.FINER)) {
                JmxProperties.MODELMBEAN_LOGGER.logp(Level.FINER, ModelMBeanInfoSupport.class.getName(), "ModelMBeanInfo(ModelMBeanInfo)", "Could not get a valid modelMBeanDescriptor, setting a default Descriptor");
            }
        }
        if (JmxProperties.MODELMBEAN_LOGGER.isLoggable(Level.FINER)) {
            JmxProperties.MODELMBEAN_LOGGER.logp(Level.FINER, ModelMBeanInfoSupport.class.getName(), "ModelMBeanInfo(ModelMBeanInfo)", "Exit");
        }
    }
    
    public ModelMBeanInfoSupport(String s, String s2, ModelMBeanAttributeInfo[] array, ModelMBeanConstructorInfo[] array2, ModelMBeanOperationInfo[] array3, ModelMBeanNotificationInfo[] array4) {
        this(s, s2, array, array2, array3, array4, null);
    }
    
    public ModelMBeanInfoSupport(String className, String description, ModelMBeanAttributeInfo[] modelMBeanAttributes, ModelMBeanConstructorInfo[] modelMBeanConstructors, ModelMBeanOperationInfo[] modelMBeanOperations, ModelMBeanNotificationInfo[] modelMBeanNotifications, Descriptor descriptor) {
        super(className, description, (modelMBeanAttributes != null) ? modelMBeanAttributes : ModelMBeanInfoSupport.NO_ATTRIBUTES, (modelMBeanConstructors != null) ? modelMBeanConstructors : ModelMBeanInfoSupport.NO_CONSTRUCTORS, (modelMBeanOperations != null) ? modelMBeanOperations : ModelMBeanInfoSupport.NO_OPERATIONS, (modelMBeanNotifications != null) ? modelMBeanNotifications : ModelMBeanInfoSupport.NO_NOTIFICATIONS);
        this.modelMBeanDescriptor = null;
        this.modelMBeanAttributes = modelMBeanAttributes;
        this.modelMBeanConstructors = modelMBeanConstructors;
        this.modelMBeanOperations = modelMBeanOperations;
        this.modelMBeanNotifications = modelMBeanNotifications;
        this.modelMBeanDescriptor = this.validDescriptor(descriptor);
        if (JmxProperties.MODELMBEAN_LOGGER.isLoggable(Level.FINER)) {
            JmxProperties.MODELMBEAN_LOGGER.logp(Level.FINER, ModelMBeanInfoSupport.class.getName(), "ModelMBeanInfoSupport(String,String,ModelMBeanAttributeInfo[],ModelMBeanConstructorInfo[],ModelMBeanOperationInfo[],ModelMBeanNotificationInfo[],Descriptor)", "Exit");
        }
    }
    
    @Override
    public Object clone() {
        return new ModelMBeanInfoSupport(this);
    }
    
    @Override
    public Descriptor[] getDescriptors(String s) throws MBeanException, RuntimeOperationsException {
        if (JmxProperties.MODELMBEAN_LOGGER.isLoggable(Level.FINER)) {
            JmxProperties.MODELMBEAN_LOGGER.logp(Level.FINER, ModelMBeanInfoSupport.class.getName(), "getDescriptors(String)", "Entry");
        }
        if (s == null || s.equals("")) {
            s = "all";
        }
        Descriptor[] array;
        if (s.equalsIgnoreCase("mbean")) {
            array = new Descriptor[] { this.modelMBeanDescriptor };
        }
        else if (s.equalsIgnoreCase("attribute")) {
            MBeanAttributeInfo[] modelMBeanAttributes2 = this.modelMBeanAttributes;
            int length8 = 0;
            if (modelMBeanAttributes2 != null) {
                length8 = modelMBeanAttributes2.length;
            }
            array = new Descriptor[length8];
            for (int n6 = 0; n6 < length8; ++n6) {
                array[n6] = ((ModelMBeanAttributeInfo)modelMBeanAttributes2[n6]).getDescriptor();
            }
        }
        else if (s.equalsIgnoreCase("operation")) {
            MBeanOperationInfo[] modelMBeanOperations2 = this.modelMBeanOperations;
            int length7 = 0;
            if (modelMBeanOperations2 != null) {
                length7 = modelMBeanOperations2.length;
            }
            array = new Descriptor[length7];
            for (int n5 = 0; n5 < length7; ++n5) {
                array[n5] = ((ModelMBeanOperationInfo)modelMBeanOperations2[n5]).getDescriptor();
            }
        }
        else if (s.equalsIgnoreCase("constructor")) {
            MBeanConstructorInfo[] modelMBeanConstructors2 = this.modelMBeanConstructors;
            int length6 = 0;
            if (modelMBeanConstructors2 != null) {
                length6 = modelMBeanConstructors2.length;
            }
            array = new Descriptor[length6];
            for (int n4 = 0; n4 < length6; ++n4) {
                array[n4] = ((ModelMBeanConstructorInfo)modelMBeanConstructors2[n4]).getDescriptor();
            }
        }
        else if (s.equalsIgnoreCase("notification")) {
            MBeanNotificationInfo[] modelMBeanNotifications2 = this.modelMBeanNotifications;
            int length5 = 0;
            if (modelMBeanNotifications2 != null) {
                length5 = modelMBeanNotifications2.length;
            }
            array = new Descriptor[length5];
            for (int n3 = 0; n3 < length5; ++n3) {
                array[n3] = ((ModelMBeanNotificationInfo)modelMBeanNotifications2[n3]).getDescriptor();
            }
        }
        else if (s.equalsIgnoreCase("all")) {
            MBeanAttributeInfo[] modelMBeanAttributes = this.modelMBeanAttributes;
            int length = 0;
            if (modelMBeanAttributes != null) {
                length = modelMBeanAttributes.length;
            }
            MBeanOperationInfo[] modelMBeanOperations = this.modelMBeanOperations;
            int length2 = 0;
            if (modelMBeanOperations != null) {
                length2 = modelMBeanOperations.length;
            }
            MBeanConstructorInfo[] modelMBeanConstructors = this.modelMBeanConstructors;
            int length3 = 0;
            if (modelMBeanConstructors != null) {
                length3 = modelMBeanConstructors.length;
            }
            MBeanNotificationInfo[] modelMBeanNotifications = this.modelMBeanNotifications;
            int length4 = 0;
            if (modelMBeanNotifications != null) {
                length4 = modelMBeanNotifications.length;
            }
            int n = length + length3 + length2 + length4 + 1;
            array = new Descriptor[n];
            array[n - 1] = this.modelMBeanDescriptor;
            int n2 = 0;
            for (int i = 0; i < length; ++i) {
                array[n2] = ((ModelMBeanAttributeInfo)modelMBeanAttributes[i]).getDescriptor();
                ++n2;
            }
            for (int j = 0; j < length3; ++j) {
                array[n2] = ((ModelMBeanConstructorInfo)modelMBeanConstructors[j]).getDescriptor();
                ++n2;
            }
            for (int k = 0; k < length2; ++k) {
                array[n2] = ((ModelMBeanOperationInfo)modelMBeanOperations[k]).getDescriptor();
                ++n2;
            }
            for (int l = 0; l < length4; ++l) {
                array[n2] = ((ModelMBeanNotificationInfo)modelMBeanNotifications[l]).getDescriptor();
                ++n2;
            }
        }
        else {
            throw new RuntimeOperationsException(new IllegalArgumentException("Descriptor Type is invalid"), "Exception occurred trying to find the descriptors of the MBean");
        }
        if (JmxProperties.MODELMBEAN_LOGGER.isLoggable(Level.FINER)) {
            JmxProperties.MODELMBEAN_LOGGER.logp(Level.FINER, ModelMBeanInfoSupport.class.getName(), "getDescriptors(String)", "Exit");
            return array;
        }
        return array;
    }
    
    @Override
    public void setDescriptors(Descriptor[] array) throws MBeanException, RuntimeOperationsException {
        if (JmxProperties.MODELMBEAN_LOGGER.isLoggable(Level.FINER)) {
            JmxProperties.MODELMBEAN_LOGGER.logp(Level.FINER, ModelMBeanInfoSupport.class.getName(), "setDescriptors(Descriptor[])", "Entry");
        }
        if (array == null) {
            throw new RuntimeOperationsException(new IllegalArgumentException("Descriptor list is invalid"), "Exception occurred trying to set the descriptors of the MBeanInfo");
        }
        if (array.length == 0) {
            return;
        }
        for (int i = 0; i < array.length; ++i) {
            this.setDescriptor(array[i], null);
        }
        if (JmxProperties.MODELMBEAN_LOGGER.isLoggable(Level.FINER)) {
            JmxProperties.MODELMBEAN_LOGGER.logp(Level.FINER, ModelMBeanInfoSupport.class.getName(), "setDescriptors(Descriptor[])", "Exit");
        }
        return;
    }
    
    public Descriptor getDescriptor(String s) throws MBeanException, RuntimeOperationsException {
        if (JmxProperties.MODELMBEAN_LOGGER.isLoggable(Level.FINER)) {
            JmxProperties.MODELMBEAN_LOGGER.logp(Level.FINER, ModelMBeanInfoSupport.class.getName(), "getDescriptor(String)", "Entry");
        }
        return this.getDescriptor(s, null);
    }
    
    @Override
    public Descriptor getDescriptor(String s, String anotherString) throws MBeanException, RuntimeOperationsException {
        if (s == null) {
            throw new RuntimeOperationsException(new IllegalArgumentException("Descriptor is invalid"), "Exception occurred trying to set the descriptors of the MBeanInfo");
        }
        if ("mbean".equalsIgnoreCase(anotherString)) {
            return (Descriptor)this.modelMBeanDescriptor.clone();
        }
        if ("attribute".equalsIgnoreCase(anotherString) || anotherString == null) {
            ModelMBeanAttributeInfo attribute = this.getAttribute(s);
            if (attribute != null) {
                return attribute.getDescriptor();
            }
            if (anotherString != null) {
                return null;
            }
        }
        if ("operation".equalsIgnoreCase(anotherString) || anotherString == null) {
            ModelMBeanOperationInfo operation = this.getOperation(s);
            if (operation != null) {
                return operation.getDescriptor();
            }
            if (anotherString != null) {
                return null;
            }
        }
        if ("constructor".equalsIgnoreCase(anotherString) || anotherString == null) {
            ModelMBeanConstructorInfo constructor = this.getConstructor(s);
            if (constructor != null) {
                return constructor.getDescriptor();
            }
            if (anotherString != null) {
                return null;
            }
        }
        if ("notification".equalsIgnoreCase(anotherString) || anotherString == null) {
            ModelMBeanNotificationInfo notification = this.getNotification(s);
            if (notification != null) {
                return notification.getDescriptor();
            }
            if (anotherString != null) {
                return null;
            }
        }
        if (anotherString == null) {
            return null;
        }
        throw new RuntimeOperationsException(new IllegalArgumentException("Descriptor Type is invalid"), "Exception occurred trying to find the descriptors of the MBean");
    }
    
    @Override
    public void setDescriptor(Descriptor descriptor, String s) throws MBeanException, RuntimeOperationsException {
        if (JmxProperties.MODELMBEAN_LOGGER.isLoggable(Level.FINER)) {
            JmxProperties.MODELMBEAN_LOGGER.logp(Level.FINER, ModelMBeanInfoSupport.class.getName(), "setDescriptor(Descriptor,String)", "Entry");
        }
        if (descriptor == null) {
            descriptor = new DescriptorSupport();
        }
        if (s == null || s.equals("")) {
            s = (String)descriptor.getFieldValue("descriptorType");
            if (s == null) {
                JmxProperties.MODELMBEAN_LOGGER.logp(Level.FINER, ModelMBeanInfoSupport.class.getName(), "setDescriptor(Descriptor,String)", "descriptorType null in both String parameter and Descriptor, defaulting to mbean");
                s = "mbean";
            }
        }
        String className = (String)descriptor.getFieldValue("name");
        if (className == null) {
            JmxProperties.MODELMBEAN_LOGGER.logp(Level.FINER, ModelMBeanInfoSupport.class.getName(), "setDescriptor(Descriptor,String)", "descriptor name null, defaulting to " + this.getClassName());
            className = this.getClassName();
        }
        boolean n = false;
        if (s.equalsIgnoreCase("mbean")) {
            this.setMBeanDescriptor(descriptor);
            n = true;
        }
        else if (s.equalsIgnoreCase("attribute")) {
            MBeanAttributeInfo[] modelMBeanAttributes = this.modelMBeanAttributes;
            int length4 = 0;
            if (modelMBeanAttributes != null) {
                length4 = modelMBeanAttributes.length;
            }
            for (int l = 0; l < length4; ++l) {
                if (className.equals(modelMBeanAttributes[l].getName())) {
                    n = true;
                    ModelMBeanAttributeInfo modelMBeanAttributeInfo = (ModelMBeanAttributeInfo)modelMBeanAttributes[l];
                    modelMBeanAttributeInfo.setDescriptor(descriptor);
                    if (JmxProperties.MODELMBEAN_LOGGER.isLoggable(Level.FINER)) {
                        JmxProperties.MODELMBEAN_LOGGER.logp(Level.FINER, ModelMBeanInfoSupport.class.getName(), "setDescriptor(Descriptor,String)", "Setting descriptor to " + descriptor + "\t\n local: AttributeInfo descriptor is " + modelMBeanAttributeInfo.getDescriptor() + "\t\n modelMBeanInfo: AttributeInfo descriptor is " + this.getDescriptor(className, "attribute"));
                    }
                }
            }
        }
        else if (s.equalsIgnoreCase("operation")) {
            MBeanOperationInfo[] modelMBeanOperations = this.modelMBeanOperations;
            int length3 = 0;
            if (modelMBeanOperations != null) {
                length3 = modelMBeanOperations.length;
            }
            for (int k = 0; k < length3; ++k) {
                if (className.equals(modelMBeanOperations[k].getName())) {
                    n = true;
                    ((ModelMBeanOperationInfo)modelMBeanOperations[k]).setDescriptor(descriptor);
                }
            }
        }
        else if (s.equalsIgnoreCase("constructor")) {
            MBeanConstructorInfo[] modelMBeanConstructors = this.modelMBeanConstructors;
            int length2 = 0;
            if (modelMBeanConstructors != null) {
                length2 = modelMBeanConstructors.length;
            }
            for (int j = 0; j < length2; ++j) {
                if (className.equals(modelMBeanConstructors[j].getName())) {
                    n = true;
                    ((ModelMBeanConstructorInfo)modelMBeanConstructors[j]).setDescriptor(descriptor);
                }
            }
        }
        else if (s.equalsIgnoreCase("notification")) {
            MBeanNotificationInfo[] modelMBeanNotifications = this.modelMBeanNotifications;
            int length = 0;
            if (modelMBeanNotifications != null) {
                length = modelMBeanNotifications.length;
            }
            for (int i = 0; i < length; ++i) {
                if (className.equals(modelMBeanNotifications[i].getName())) {
                    n = true;
                    ((ModelMBeanNotificationInfo)modelMBeanNotifications[i]).setDescriptor(descriptor);
                }
            }
        }
        else {
            throw new RuntimeOperationsException(new IllegalArgumentException("Invalid descriptor type: " + s), "Exception occurred trying to set the descriptors of the MBean");
        }
        if (n == 0) {
            throw new RuntimeOperationsException(new IllegalArgumentException("Descriptor name is invalid: type=" + s + "; name=" + className), "Exception occurred trying to set the descriptors of the MBean");
        }
        if (JmxProperties.MODELMBEAN_LOGGER.isLoggable(Level.FINER)) {
            JmxProperties.MODELMBEAN_LOGGER.logp(Level.FINER, ModelMBeanInfoSupport.class.getName(), "setDescriptor(Descriptor,String)", "Exit");
        }
        return;
    }
    
    @Override
    public ModelMBeanAttributeInfo getAttribute(String s) throws MBeanException, RuntimeOperationsException {
        ModelMBeanAttributeInfo modelMBeanAttributeInfo = null;
        if (JmxProperties.MODELMBEAN_LOGGER.isLoggable(Level.FINER)) {
            JmxProperties.MODELMBEAN_LOGGER.logp(Level.FINER, ModelMBeanInfoSupport.class.getName(), "getAttribute(String)", "Entry");
        }
        if (s == null) {
            throw new RuntimeOperationsException(new IllegalArgumentException("Attribute Name is null"), "Exception occurred trying to get the ModelMBeanAttributeInfo of the MBean");
        }
        MBeanAttributeInfo[] modelMBeanAttributes = this.modelMBeanAttributes;
        int length = 0;
        if (modelMBeanAttributes != null) {
            length = modelMBeanAttributes.length;
        }
        for (int n = 0; n < length && modelMBeanAttributeInfo == null; ++n) {
            if (JmxProperties.MODELMBEAN_LOGGER.isLoggable(Level.FINER)) {
                JmxProperties.MODELMBEAN_LOGGER.logp(Level.FINER, ModelMBeanInfoSupport.class.getName(), "getAttribute(String)", "\t\n this.getAttributes() MBeanAttributeInfo Array " + n + ":" + ((ModelMBeanAttributeInfo)modelMBeanAttributes[n]).getDescriptor() + "\t\n this.modelMBeanAttributes MBeanAttributeInfo Array " + n + ":" + ((ModelMBeanAttributeInfo)this.modelMBeanAttributes[n]).getDescriptor());
            }
            if (s.equals(modelMBeanAttributes[n].getName())) {
                modelMBeanAttributeInfo = (ModelMBeanAttributeInfo)modelMBeanAttributes[n].clone();
            }
        }
        if (JmxProperties.MODELMBEAN_LOGGER.isLoggable(Level.FINER)) {
            JmxProperties.MODELMBEAN_LOGGER.logp(Level.FINER, ModelMBeanInfoSupport.class.getName(), "getAttribute(String)", "Exit");
            return modelMBeanAttributeInfo;
        }
        return modelMBeanAttributeInfo;
    }
    
    @Override
    public ModelMBeanOperationInfo getOperation(String s) throws MBeanException, RuntimeOperationsException {
        ModelMBeanOperationInfo modelMBeanOperationInfo = null;
        if (JmxProperties.MODELMBEAN_LOGGER.isLoggable(Level.FINER)) {
            JmxProperties.MODELMBEAN_LOGGER.logp(Level.FINER, ModelMBeanInfoSupport.class.getName(), "getOperation(String)", "Entry");
        }
        if (s == null) {
            throw new RuntimeOperationsException(new IllegalArgumentException("inName is null"), "Exception occurred trying to get the ModelMBeanOperationInfo of the MBean");
        }
        MBeanOperationInfo[] modelMBeanOperations = this.modelMBeanOperations;
        int length = 0;
        if (modelMBeanOperations != null) {
            length = modelMBeanOperations.length;
        }
        for (int n = 0; n < length && modelMBeanOperationInfo == null; ++n) {
            if (s.equals(modelMBeanOperations[n].getName())) {
                modelMBeanOperationInfo = (ModelMBeanOperationInfo)modelMBeanOperations[n].clone();
            }
        }
        if (JmxProperties.MODELMBEAN_LOGGER.isLoggable(Level.FINER)) {
            JmxProperties.MODELMBEAN_LOGGER.logp(Level.FINER, ModelMBeanInfoSupport.class.getName(), "getOperation(String)", "Exit");
            return modelMBeanOperationInfo;
        }
        return modelMBeanOperationInfo;
    }
    
    public ModelMBeanConstructorInfo getConstructor(String s) throws MBeanException, RuntimeOperationsException {
        ModelMBeanConstructorInfo modelMBeanConstructorInfo = null;
        if (JmxProperties.MODELMBEAN_LOGGER.isLoggable(Level.FINER)) {
            JmxProperties.MODELMBEAN_LOGGER.logp(Level.FINER, ModelMBeanInfoSupport.class.getName(), "getConstructor(String)", "Entry");
        }
        if (s == null) {
            throw new RuntimeOperationsException(new IllegalArgumentException("Constructor name is null"), "Exception occurred trying to get the ModelMBeanConstructorInfo of the MBean");
        }
        MBeanConstructorInfo[] modelMBeanConstructors = this.modelMBeanConstructors;
        int length = 0;
        if (modelMBeanConstructors != null) {
            length = modelMBeanConstructors.length;
        }
        for (int n = 0; n < length && modelMBeanConstructorInfo == null; ++n) {
            if (s.equals(modelMBeanConstructors[n].getName())) {
                modelMBeanConstructorInfo = (ModelMBeanConstructorInfo)modelMBeanConstructors[n].clone();
            }
        }
        if (JmxProperties.MODELMBEAN_LOGGER.isLoggable(Level.FINER)) {
            JmxProperties.MODELMBEAN_LOGGER.logp(Level.FINER, ModelMBeanInfoSupport.class.getName(), "getConstructor(String)", "Exit");
            return modelMBeanConstructorInfo;
        }
        return modelMBeanConstructorInfo;
    }
    
    @Override
    public ModelMBeanNotificationInfo getNotification(String s) throws MBeanException, RuntimeOperationsException {
        ModelMBeanNotificationInfo modelMBeanNotificationInfo = null;
        if (JmxProperties.MODELMBEAN_LOGGER.isLoggable(Level.FINER)) {
            JmxProperties.MODELMBEAN_LOGGER.logp(Level.FINER, ModelMBeanInfoSupport.class.getName(), "getNotification(String)", "Entry");
        }
        if (s == null) {
            throw new RuntimeOperationsException(new IllegalArgumentException("Notification name is null"), "Exception occurred trying to get the ModelMBeanNotificationInfo of the MBean");
        }
        MBeanNotificationInfo[] modelMBeanNotifications = this.modelMBeanNotifications;
        int length = 0;
        if (modelMBeanNotifications != null) {
            length = modelMBeanNotifications.length;
        }
        for (int n = 0; n < length && modelMBeanNotificationInfo == null; ++n) {
            if (s.equals(modelMBeanNotifications[n].getName())) {
                modelMBeanNotificationInfo = (ModelMBeanNotificationInfo)modelMBeanNotifications[n].clone();
            }
        }
        if (JmxProperties.MODELMBEAN_LOGGER.isLoggable(Level.FINER)) {
            JmxProperties.MODELMBEAN_LOGGER.logp(Level.FINER, ModelMBeanInfoSupport.class.getName(), "getNotification(String)", "Exit");
            return modelMBeanNotificationInfo;
        }
        return modelMBeanNotificationInfo;
    }
    
    @Override
    public Descriptor getDescriptor() {
        return this.getMBeanDescriptorNoException();
    }
    
    @Override
    public Descriptor getMBeanDescriptor() throws MBeanException {
        return this.getMBeanDescriptorNoException();
    }
    
    private Descriptor getMBeanDescriptorNoException() {
        if (JmxProperties.MODELMBEAN_LOGGER.isLoggable(Level.FINER)) {
            JmxProperties.MODELMBEAN_LOGGER.logp(Level.FINER, ModelMBeanInfoSupport.class.getName(), "getMBeanDescriptorNoException()", "Entry");
        }
        if (this.modelMBeanDescriptor == null) {
            this.modelMBeanDescriptor = this.validDescriptor(null);
        }
        if (JmxProperties.MODELMBEAN_LOGGER.isLoggable(Level.FINER)) {
            JmxProperties.MODELMBEAN_LOGGER.logp(Level.FINER, ModelMBeanInfoSupport.class.getName(), "getMBeanDescriptorNoException()", "Exit, returning: " + this.modelMBeanDescriptor);
        }
        return (Descriptor)this.modelMBeanDescriptor.clone();
    }
    
    @Override
    public void setMBeanDescriptor(Descriptor descriptor) throws MBeanException, RuntimeOperationsException {
        if (JmxProperties.MODELMBEAN_LOGGER.isLoggable(Level.FINER)) {
            JmxProperties.MODELMBEAN_LOGGER.logp(Level.FINER, ModelMBeanInfoSupport.class.getName(), "setMBeanDescriptor(Descriptor)", "Entry");
        }
        this.modelMBeanDescriptor = this.validDescriptor(descriptor);
    }
    
    private Descriptor validDescriptor(Descriptor descriptor) throws RuntimeOperationsException {
        int n = descriptor == null;
        DescriptorSupport descriptorSupport;
        if (!(n == 0)) {
            descriptorSupport = new DescriptorSupport();
            JmxProperties.MODELMBEAN_LOGGER.finer("Null Descriptor, creating new.");
        }
        else {
            descriptorSupport = (Descriptor)descriptor.clone();
        }
        if (n && descriptorSupport.getFieldValue("name") == null) {
            descriptorSupport.setField("name", this.getClassName());
            JmxProperties.MODELMBEAN_LOGGER.finer("Defaulting Descriptor name to " + this.getClassName());
        }
        if (n && descriptorSupport.getFieldValue("descriptorType") == null) {
            descriptorSupport.setField("descriptorType", "mbean");
            JmxProperties.MODELMBEAN_LOGGER.finer("Defaulting descriptorType to \"mbean\"");
        }
        if (descriptorSupport.getFieldValue("displayName") == null) {
            descriptorSupport.setField("displayName", this.getClassName());
            JmxProperties.MODELMBEAN_LOGGER.finer("Defaulting Descriptor displayName to " + this.getClassName());
        }
        if (descriptorSupport.getFieldValue("persistPolicy") == null) {
            descriptorSupport.setField("persistPolicy", "never");
            JmxProperties.MODELMBEAN_LOGGER.finer("Defaulting Descriptor persistPolicy to \"never\"");
        }
        if (descriptorSupport.getFieldValue("log") == null) {
            descriptorSupport.setField("log", "F");
            JmxProperties.MODELMBEAN_LOGGER.finer("Defaulting Descriptor \"log\" field to \"F\"");
        }
        if (descriptorSupport.getFieldValue("visibility") == null) {
            descriptorSupport.setField("visibility", "1");
            JmxProperties.MODELMBEAN_LOGGER.finer("Defaulting Descriptor visibility to 1");
        }
        if (!descriptorSupport.isValid()) {
            throw new RuntimeOperationsException(new IllegalArgumentException("Invalid Descriptor argument"), "The isValid() method of the Descriptor object itself returned false,one or more required fields are invalid. Descriptor:" + descriptorSupport.toString());
        }
        if (!((String)descriptorSupport.getFieldValue("descriptorType")).equalsIgnoreCase("mbean")) {
            throw new RuntimeOperationsException(new IllegalArgumentException("Invalid Descriptor argument"), "The Descriptor \"descriptorType\" field does not match the object described.  Expected: mbean , was: " + descriptorSupport.getFieldValue("descriptorType"));
        }
        return descriptorSupport;
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        if (ModelMBeanInfoSupport.compat) {
            ObjectInputStream.GetField fields = objectInputStream.readFields();
            this.modelMBeanDescriptor = (Descriptor)fields.get("modelMBeanDescriptor", null);
            if (fields.defaulted("modelMBeanDescriptor")) {
                throw new NullPointerException("modelMBeanDescriptor");
            }
            this.modelMBeanAttributes = (MBeanAttributeInfo[])(MBeanAttributeInfo[])fields.get("mmbAttributes", null);
            if (fields.defaulted("mmbAttributes")) {
                throw new NullPointerException("mmbAttributes");
            }
            this.modelMBeanConstructors = (MBeanConstructorInfo[])(MBeanConstructorInfo[])fields.get("mmbConstructors", null);
            if (fields.defaulted("mmbConstructors")) {
                throw new NullPointerException("mmbConstructors");
            }
            this.modelMBeanNotifications = (MBeanNotificationInfo[])(MBeanNotificationInfo[])fields.get("mmbNotifications", null);
            if (fields.defaulted("mmbNotifications")) {
                throw new NullPointerException("mmbNotifications");
            }
            this.modelMBeanOperations = (MBeanOperationInfo[])(MBeanOperationInfo[])fields.get("mmbOperations", null);
            if (fields.defaulted("mmbOperations")) {
                throw new NullPointerException("mmbOperations");
            }
        }
        else {
            objectInputStream.defaultReadObject();
        }
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        if (ModelMBeanInfoSupport.compat) {
            ObjectOutputStream.PutField putFields = objectOutputStream.putFields();
            putFields.put("modelMBeanDescriptor", this.modelMBeanDescriptor);
            putFields.put("mmbAttributes", this.modelMBeanAttributes);
            putFields.put("mmbConstructors", this.modelMBeanConstructors);
            putFields.put("mmbNotifications", this.modelMBeanNotifications);
            putFields.put("mmbOperations", this.modelMBeanOperations);
            putFields.put("currClass", "ModelMBeanInfoSupport");
            objectOutputStream.writeFields();
        }
        else {
            objectOutputStream.defaultWriteObject();
        }
    }
    
    static {
        oldSerialPersistentFields = new ObjectStreamField[] { new ObjectStreamField("modelMBeanDescriptor", Descriptor.class), new ObjectStreamField("mmbAttributes", MBeanAttributeInfo[].class), new ObjectStreamField("mmbConstructors", MBeanConstructorInfo[].class), new ObjectStreamField("mmbNotifications", MBeanNotificationInfo[].class), new ObjectStreamField("mmbOperations", MBeanOperationInfo[].class), new ObjectStreamField("currClass", String.class) };
        newSerialPersistentFields = new ObjectStreamField[] { new ObjectStreamField("modelMBeanDescriptor", Descriptor.class), new ObjectStreamField("modelMBeanAttributes", MBeanAttributeInfo[].class), new ObjectStreamField("modelMBeanConstructors", MBeanConstructorInfo[].class), new ObjectStreamField("modelMBeanNotifications", MBeanNotificationInfo[].class), new ObjectStreamField("modelMBeanOperations", MBeanOperationInfo[].class) };
        ModelMBeanInfoSupport.compat = false;
        try {
            String s = (String)AccessController.doPrivileged(new GetPropertyAction("jmx.serial.form"));
            ModelMBeanInfoSupport.compat = (s != null && s.equals("1.0"));
        }
        catch (Exception ex) {}
        if (ModelMBeanInfoSupport.compat) {
            serialPersistentFields = ModelMBeanInfoSupport.oldSerialPersistentFields;
            serialVersionUID = -3944083498453227709L;
        }
        else {
            serialPersistentFields = ModelMBeanInfoSupport.newSerialPersistentFields;
            serialVersionUID = -1935722590756516193L;
        }
        NO_ATTRIBUTES = new ModelMBeanAttributeInfo[0];
        NO_CONSTRUCTORS = new ModelMBeanConstructorInfo[0];
        NO_NOTIFICATIONS = new ModelMBeanNotificationInfo[0];
        NO_OPERATIONS = new ModelMBeanOperationInfo[0];
    }
}
